@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.27 2002/03/25 12:44:35 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.28 2002/04/03 14:19:01 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -1687,7 +1687,10 @@ do_ip_vs_set_ctl(struct sock *sk, int cmd, void *user, unsigned int len)
 	}
 
 	MOD_INC_USE_COUNT;
-	down_interruptible(&__ip_vs_mutex);
+	if (down_interruptible(&__ip_vs_mutex)) {
+		ret = -ERESTARTSYS;
+		goto out_dec;
+	}
 
 	if (cmd == IP_VS_SO_SET_FLUSH) {
 		/* Flush the virtual service */
@@ -1771,10 +1774,11 @@ do_ip_vs_set_ctl(struct sock *sk, int cmd, void *user, unsigned int len)
 	if (svc)
 		ip_vs_service_put(svc);
 
- out_unlock:
+  out_unlock:
 	up(&__ip_vs_mutex);
+  out_dec:
 	MOD_DEC_USE_COUNT;
- out_free:
+  out_free:
 	kfree(urule);
 	return ret;
 }
@@ -1901,7 +1905,8 @@ do_ip_vs_get_ctl(struct sock *sk, int cmd, void *user, int *len)
 	if (!capable(CAP_NET_ADMIN))
 		return -EPERM;
 
-	down_interruptible(&__ip_vs_mutex);
+	if (down_interruptible(&__ip_vs_mutex))
+		return -ERESTARTSYS;
 
 	switch (cmd) {
 	case IP_VS_SO_GET_VERSION:
