@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_sched.c,v 1.10 2001/06/04 13:51:03 wensong Exp $
+ * Version:     $Id: ip_vs_sched.c,v 1.11 2001/11/04 08:58:43 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -146,7 +146,7 @@ static struct ip_vs_scheduler *ip_vs_sched_getbyname(const char *sched_name)
 /*
  *  Lookup scheduler and try to load it if it doesn't exist
  */
-struct ip_vs_scheduler *ip_vs_lookup_scheduler(const char *sched_name)
+struct ip_vs_scheduler *ip_vs_scheduler_get(const char *sched_name)
 {
 	struct ip_vs_scheduler *sched;
 
@@ -168,6 +168,12 @@ struct ip_vs_scheduler *ip_vs_lookup_scheduler(const char *sched_name)
 	return sched;
 }
 
+void ip_vs_scheduler_put(struct ip_vs_scheduler *scheduler)
+{
+	if (scheduler->module)
+		__MOD_DEC_USE_COUNT(scheduler->module);
+}
+
 
 /*
  *  Register a scheduler in the scheduler list
@@ -194,8 +200,7 @@ int register_ip_vs_scheduler(struct ip_vs_scheduler *scheduler)
 	 */
 	sched = ip_vs_sched_getbyname(scheduler->name);
 	if (sched) {
-		if (sched->module)
-			__MOD_DEC_USE_COUNT(sched->module);
+		ip_vs_scheduler_put(sched);
 		MOD_DEC_USE_COUNT;
 		IP_VS_ERR("register_ip_vs_scheduler(): [%s] scheduler "
 			  "already existed in the system\n", scheduler->name);
