@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_sync.c,v 1.4 2001/11/04 08:58:43 wensong Exp $
+ * Version:     $Id: ip_vs_sync.c,v 1.5 2001/12/11 12:41:29 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -19,7 +19,7 @@
 #include <linux/module.h>
 #include <linux/kernel.h>
 #include <linux/errno.h>
-#include <linux/malloc.h>
+#include <linux/slab.h>
 #include <linux/net.h>
 #include <linux/sched.h>
 #include <linux/wait.h>
@@ -43,9 +43,11 @@
  *	IPVS sync connection entry
  */
 struct ip_vs_sync_conn {
+	__u8			reserved;
+
 	/* Protocol, addresses and port numbers */
-	__u16                   protocol;       /* Which protocol (TCP/UDP) */
-	__u16                   cport;
+	__u8			protocol;       /* Which protocol (TCP/UDP) */
+	__u16			cport;
 	__u16                   vport;
 	__u16                   dport;
 	__u32                   caddr;          /* client address */
@@ -202,8 +204,8 @@ void ip_vs_sync_conn(struct ip_vs_conn *cp)
 	s->caddr = cp->caddr;
 	s->vaddr = cp->vaddr;
 	s->daddr = cp->daddr;
-	s->flags = cp->flags & ~IP_VS_CONN_F_HASHED;
-	s->state = cp->state;
+	s->flags = htons(cp->flags & ~IP_VS_CONN_F_HASHED);
+	s->state = htons(cp->state);
 	if (cp->flags & IP_VS_CONN_F_SEQ_MASK) {
 		struct ip_vs_sync_conn_options *opt =
 			(struct ip_vs_sync_conn_options *)&s[1];
@@ -220,6 +222,10 @@ void ip_vs_sync_conn(struct ip_vs_conn *cp)
 		curr_sb = NULL;
 	}
 	spin_unlock(&curr_sb_lock);
+
+	/* synchronize its controller if it has */
+	if (cp->control)
+		ip_vs_sync_conn(cp->control);
 }
 
 
@@ -252,15 +258,15 @@ static void ip_vs_process_message(const char *buffer, const size_t buflen)
 					    s->caddr, s->cport,
 					    s->vaddr, s->vport,
 					    s->daddr, s->dport,
-					    s->flags, NULL);
+					    ntohs(s->flags), NULL);
 			if (!cp) {
 				IP_VS_ERR("ip_vs_conn_new failed\n");
 				return;
 			}
 		} else
-			cp->flags = s->flags | IP_VS_CONN_F_HASHED;
+			cp->flags = ntohs(s->flags) | IP_VS_CONN_F_HASHED;
 
-		cp->state = s->state;
+		cp->state = ntohs(s->state);
 		if (cp->flags & IP_VS_CONN_F_SEQ_MASK) {
 			opt = (struct ip_vs_sync_conn_options *)&s[1];
 			memcpy(&cp->in_seq, opt, sizeof(*opt));
@@ -693,6 +699,8 @@ int start_sync_thread(int state, char *mcast_ifn)
 	if (sync_pid)
 		return -EEXIST;
 
+	IP_VS_DBG(7, "Each ip_vs_sync_conn entry need %d bytes\n",
+		  sizeof(struct ip_vs_sync_conn));
 	ip_vs_sync_state = state;
 	strcpy(ip_vs_mcast_ifn, mcast_ifn);
 
