@@ -5,10 +5,10 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_timer.c,v 1.7 2002/01/24 14:23:49 wensong Exp $
+ * Version:     $Id: ip_vs_timer.c,v 1.8 2002/01/31 14:33:12 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
- *              Julian Anastasov <uli@linux.tu-varna.acad.bg>
+ *              Julian Anastasov <ja@ssi.bg>
  *
  *              This program is free software; you can redistribute it and/or
  *              modify it under the terms of the GNU General Public License
@@ -79,13 +79,13 @@ static inline void internal_add_sltimer(struct timer_list *timer)
 	 * must hold the sltimer lock when calling this
 	 */
 	unsigned long expires = timer->expires;
-	unsigned long idx = (expires - sltimer_jiffies) >> SHIFT_BITS;
+	unsigned long idx = expires - sltimer_jiffies;
 	struct list_head * vec;
 
-	if (idx < TVR_SIZE) {
+	if (idx < 1 << (SHIFT_BITS + TVR_BITS)) {
 		int i = (expires >> SHIFT_BITS) & TVR_MASK;
 		vec = sltv1.vec + i;
-	} else if (idx < 1 << (TVR_BITS + TVN_BITS)) {
+	} else if (idx < 1 << (SHIFT_BITS + TVR_BITS + TVN_BITS)) {
 		int i = (expires >> (SHIFT_BITS+TVR_BITS)) & TVN_MASK;
 		vec = sltv2.vec + i;
 	} else if ((signed long) idx < 0) {
